@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: toolbox.c 8895 2008-12-28 12:19:29Z yeti-dn $
+ *  @(#) $Id: toolbox.c 10264 2009-10-19 20:40:47Z yeti-dn $
  *  Copyright (C) 2003-2006 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -32,10 +32,7 @@
 #include "gwyappinternal.h"
 #include "gwyddion.h"
 
-#ifdef HAVE_REMOTE_WIN32
-#include <gdk/gdkwin32.h>
-#include <shlobj.h>
-#endif
+#include "mac_integration.h"
 
 enum {
     DND_TARGET_STRING = 1,
@@ -94,11 +91,6 @@ static void       gwy_app_tool_use_cb          (const gchar *toolname,
                                                 GtkWidget *button);
 static void gwy_app_change_default_mask_color_cb(void);
 static void       gwy_app_gl_view_maybe_cb     (void);
-#ifdef HAVE_REMOTE_WIN32
-static void       toolbox_destroy              (GtkWidget *widget,
-                                                GdkEvent *event,
-                                                gpointer user_data);
-#endif
 
 static GtkTargetEntry dnd_target_table[] = {
     { "STRING",        0, DND_TARGET_STRING, },
@@ -585,10 +577,6 @@ gwy_app_toolbox_create(void)
     GtkBox *vbox;
     GtkTooltips *tooltips;
     GtkAccelGroup *accel_group;
-#ifdef HAVE_REMOTE_WIN32
-    GdkWindow *gdk_win;
-    HWND hwnd = 0;
-#endif
 
     toolbox = gtk_window_new(GTK_WINDOW_TOPLEVEL);
     gtk_window_set_title(GTK_WINDOW(toolbox), g_get_application_name());
@@ -636,24 +624,16 @@ gwy_app_toolbox_create(void)
     /***************************************************************/
     /* XXX */
     g_signal_connect(toolbox, "delete-event", G_CALLBACK(gwy_app_quit), NULL);
-#ifdef HAVE_REMOTE_WIN32
-    g_signal_connect(toolbox, "destroy", G_CALLBACK(toolbox_destroy), NULL);
-#endif
 
     gtk_window_add_accel_group(GTK_WINDOW(toolbox), accel_group);
     gtk_widget_show_all(toolbox);
 
+    gwy_osx_get_menu_from_widget(container);
     while (gtk_events_pending())
         gtk_main_iteration_do(FALSE);
 
-#ifdef HAVE_REMOTE_WIN32
-    gdk_win = gtk_widget_get_root_window(toolbox);
-    // Retrieve hWnd using gdk window
-    hwnd = GDK_WINDOW_HWND(toolbox->window);
-    // Create property and set it to 1
-    SetProp(hwnd, GWY_TOOLBOX_WM_ROLE, (HANDLE)1);
-    gwy_debug("SetProp to hWnd %d\n", hwnd);
-#endif
+    gwy_remote_setup(toolbox);
+    g_signal_connect(toolbox, "destroy", G_CALLBACK(gwy_remote_finalize), NULL);
 
     return toolbox;
 }
@@ -1028,22 +1008,6 @@ toolbox_dnd_data_received(G_GNUC_UNUSED GtkWidget *widget,
         g_ptr_array_free(files, TRUE);
 }
 
-#ifdef HAVE_REMOTE_WIN32
-static void
-toolbox_destroy(GtkWidget *widget,
-                GdkEvent *event,
-                gpointer user_data)
-{
-    HWND hwnd = 0;
-
-    // Retrieve hWnd
-    hwnd = GDK_WINDOW_HWND(gtk_widget_get_root_window(widget));
-    // Remove properties
-    RemoveProp(hwnd, GWY_TOOLBOX_WM_ROLE);
-    gwy_debug("RemoveProp to hWnd %d\n", hwnd);
-}
-#endif
-
 static void
 gwy_app_meta_browser(void)
 {
@@ -1172,4 +1136,6 @@ gwy_app_gl_view_maybe_cb(void)
     gtk_widget_show(dialog);
 }
 
+
+
 /* vim: set cin et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
