@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: gwyutils.c 9172 2009-05-18 17:44:34Z yeti-dn $
+ *  @(#) $Id: gwyutils.c 9596 2009-08-11 20:02:51Z yeti-dn $
  *  Copyright (C) 2003-2004 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -53,6 +53,10 @@
 G_WIN32_DLLMAIN_FOR_DLL_NAME(static, dll_name);
 #endif
 
+#ifdef __APPLE__
+#include <CoreFoundation/CoreFoundation.h>
+#endif
+
 static GQuark error_domain = 0;
 
 /**
@@ -500,6 +504,52 @@ gwy_sgettext(const gchar *msgid)
     return msgstr;
 }
 
+#ifdef __APPLE__
+static gchar *
+gwy_osx_find_dir_in_bundle(const gchar *dirname)
+{
+    static gchar *basedir = NULL;
+
+    if (!basedir) {
+        int maxlen = 256;
+        int len;
+        char *res_url_path;
+
+        CFURLRef res_url_ref = NULL, bundle_url_ref = NULL;
+
+        res_url_ref =
+            CFBundleCopyResourcesDirectoryURL(CFBundleGetMainBundle());
+        bundle_url_ref = CFBundleCopyBundleURL(CFBundleGetMainBundle());
+
+        if (res_url_ref
+            && bundle_url_ref && !CFEqual(res_url_ref, bundle_url_ref)) {
+
+            res_url_path = malloc(maxlen);
+
+            while (!CFURLGetFileSystemRepresentation(res_url_ref, true,
+                                                     (UInt8*) res_url_path,
+                                                     maxlen)) {
+                maxlen *= 2;
+                res_url_path = realloc(res_url_path, maxlen);
+            }
+
+            len = strlen(res_url_path);
+            basedir = malloc(len + 1);
+            strncpy(basedir, res_url_path, len);
+            basedir[len] = 0;
+        }
+        if (res_url_ref)
+            CFRelease(res_url_ref);
+        if (bundle_url_ref)
+            CFRelease(bundle_url_ref);
+    }
+    if (gwy_strequal(dirname, "data"))
+        dirname = NULL;
+
+    return basedir ? g_build_filename(basedir, dirname, NULL) : NULL;
+}
+#endif
+
 /**
  * gwy_find_self_dir:
  * @dirname: A gwyddion directory name:
@@ -539,6 +589,7 @@ gwy_find_self_dir(const gchar *dirname)
 #endif    /* G_OS_WIN32 */
 
 #ifdef G_OS_UNIX
+#ifdef GWY_LIBDIR
     static const struct {
         const gchar *id;
         const gchar *base;
@@ -579,7 +630,14 @@ gwy_find_self_dir(const gchar *dirname)
     };
     gsize i;
     const gchar *base;
+#endif    /* GWY_LIBDIR */
+#ifdef __APPLE__
+    char *ret = gwy_osx_find_dir_in_bundle(dirname);
 
+    if (ret)
+        return ret;
+#endif    /* __APPLE__ */
+#ifdef GWY_LIBDIR
     for (i = 0; i < G_N_ELEMENTS(paths); i++) {
         if (!gwy_strequal(dirname, paths[i].id))
             continue;
@@ -592,6 +650,7 @@ gwy_find_self_dir(const gchar *dirname)
         return g_build_filename(base, paths[i].dir, NULL);
     }
     g_critical("Cannot find directory for `%s'", dirname);
+#endif    /* GWY_LIBDIR */
     return NULL;
 #endif    /* G_OS_UNIX */
 }
@@ -801,15 +860,32 @@ gwy_filename_ignore(const gchar *filename_sys)
 
 /**
  * gwy_str_next_line:
- * @buffer: A character buffer containing some text.
+ * @buffer: Text buffer.
  *
  * Extracts a next line from a character buffer, modifying it in place.
  *
  * @buffer is updated to point after the end of the line and the "\n"
- * (or "\r\n") is replaced with "\0", if present.
+ * (or "\r" or "\r\n") is replaced with "\0", if present.
+ *
+ * The final line may or may not be terminated with an EOL marker, its contents
+ * is returned in either case.  Note, however, that the empty string "" is not
+ * interpreted as an empty unterminated line.  Instead, %NULL is immediately
+ * returned.
+ *
+ * The typical usage of gwy_str_next_line() is:
+ * |[
+ * gchar *p = text;
+ * for (gchar *line = gwy_str_next_line(&p);
+ *      line;
+ *      line = gwy_str_next_line(&p)) {
+ *     g_strstrip(line);
+ *     // Do something more with line
+ * }
+ * ]|
  *
  * Returns: The start of the line.  %NULL if the buffer is empty or %NULL.
- *          NOT a new string, the returned pointer points somewhere to @buffer.
+ *          The return value is <emphasis>not</emphasis> a new string; the
+ *          normal return value is the previous value of @buffer.
  **/
 gchar*
 gwy_str_next_line(gchar **buffer)
@@ -820,16 +896,22 @@ gwy_str_next_line(gchar **buffer)
         return NULL;
 
     q = *buffer;
-    p = strchr(*buffer, '\n');
-    if (p) {
-        if (p > *buffer && *(p-1) == '\r')
-            *(p-1) = '\0';
-        *buffer = p+1;
-        *p = '\0';
-    }
-    else
+    if (!*q) {
         *buffer = NULL;
+        return NULL;
+    }
 
+    for (p = q; *p != '\n' && *p != '\r' && *p; p++)
+        ;
+    if (p[0] == '\r' && p[1] == '\n') {
+        *(p++) = '\0';
+        *(p++) = '\0';
+    }
+    else if (p[0]) {
+        *(p++) = '\0';
+    }
+
+    *buffer = p;
     return q;
 }
 
