@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: filters.c 8874 2008-12-28 12:16:22Z yeti-dn $
+ *  @(#) $Id: filters.c 11395 2010-09-15 12:51:24Z yeti-dn $
  *  Copyright (C) 2003 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -294,8 +294,8 @@ gwy_data_field_clamp(GwyDataField *data_field,
     if (tot) {
         /* We can precompute stats */
         data_field->cached &= CBIT(MIN) | CBIT(MAX) | CBIT(MED);
-        CVAL(data_field, MIN) = MIN(bottom, CVAL(data_field, MIN));
-        CVAL(data_field, MAX) = MAX(top, CVAL(data_field, MAX));
+        CVAL(data_field, MIN) = MAX(bottom, CVAL(data_field, MIN));
+        CVAL(data_field, MAX) = MIN(top, CVAL(data_field, MAX));
         if (CTEST(data_field, MED)
             && (CVAL(data_field, MED) < bottom || CVAL(data_field, MED) > top))
             data_field->cached &= ~CBIT(MED);
