@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: facet_analysis.c 8263 2007-06-25 21:21:28Z yeti-dn $
+ *  @(#) $Id: facet_analysis.c 8838 2008-12-12 14:25:33Z yeti-dn $
  *  Copyright (C) 2003 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -395,7 +395,7 @@ facets_dialog(FacetsArgs *args,
 
     controls.tolerance = gtk_adjustment_new(args->tolerance*180.0/G_PI,
                                             0.0, 15.0, 0.01, 0.1, 0);
-    scale = gwy_table_attach_hscale(table, row++, _("_Tolerance:"), "deg",
+    scale = gwy_table_attach_hscale(table, row++, _("_Tolerance:"), _("deg"),
                                     controls.tolerance, 0);
     gtk_spin_button_set_digits(GTK_SPIN_BUTTON(scale), 3);
     g_signal_connect(controls.tolerance, "value-changed",
