@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: scale.c 9105 2009-03-30 12:00:25Z yeti-dn $
+ *  @(#) $Id: scale.c 11394 2010-09-15 12:01:39Z yeti-dn $
  *  Copyright (C) 2003 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -102,7 +102,7 @@ module_register(void)
 {
     gwy_process_func_register("scale",
                               (GwyProcessFunc)&scale,
-                              N_("/_Basic Operations/Scale..."),
+                              N_("/_Basic Operations/_Scale..."),
                               GWY_STOCK_SCALE,
                               SCALE_RUN_MODES,
                               GWY_MENU_FLAG_DATA,
