@@ -1,6 +1,20 @@
 #!/usr/bin/python
+# Script for generation of gwy.py dummy file. The file contains empty function bodies with comments.
+# Used for generation of documentation by using epydoc.
+# Public domain
 
-import re, sys
+import re, sys, string
+# interesting variables
+doc_dirs = string.split("../../app ../../libdraw ../../libgwyddion ../../libgwydgets ../../libgwymodule ../../libprocess .")
+defs_file = "pygwy.defs"
+wrap_file = "pygwywrap.c"
+ignore_functions = ["static", "typedef", "be", "parameter"]
+ignore_endswith = ["_wrap"]
+datatype = {'gdouble': "float", 'GDoubleValue': "float", 'gboolean': "bool", 'const-gchar*': "string", 'const-guchar*': "string", \
+   'gint': "int", 'guint': "int", 'gint32': "int", 'GIntValue': "int", 'GwyRGBA*': "L{RGBA}", 'GQuark': "int", 'GwyContainer*': "L{Container}", \
+   'GwyDataField*': "L{DataField}", 'GwyDataLine*': "L{DataLine}", 'GwyGraphModel*': "L{GraphModel}", 'GwySIUnit*': "L{SIUnit}", 'GwyGraph*': "L{Graph}", \
+   'GwySpectra*': "L{Spectra}", 'GwyDataView*': "L{DataView}", 'GtkWidget*': "L{gtk.Widget}", 'GObject*': "L{gobject.GObject}", 'GtkWindow*': "L{gtk.Window}", \
+   'GtkTooltips*': "L{gtk.Tooltips}", 'GtkAccelGroup*': "L{gtk.AccelGroup}", 'GtkComboBox*': "L{gtk.ComboBox}"}
 
 
 def get_python_name(c_name):
@@ -65,7 +79,14 @@ def printdoc(s):
 
    print replace_special(s)
 
+def replace_datatype(s):   
+   if datatype.has_key(s):
+      return datatype[s]
+   else:
+      return s
+
 def print_functions(spaces, method, docs, enums, level=1):
+   ignored_params = []
    enum_str = dict()
    spaces2 = spaces
    if level == 0:
@@ -73,15 +94,20 @@ def print_functions(spaces, method, docs, enums, level=1):
    print spaces+"def", method.name+"(",
 
    for i in range(len(method.params)):
+      if method.params[i].ptype.startswith("GError"):
+         ignored_params.append(method.params[i].pname)
+         continue
+      if i > 0:
+         print ',',
       # from is a keyword in Python
       method.params[i].pname = method.params[i].pname.replace('[', '')
       method.params[i].pname = method.params[i].pname.replace(']', '')
       if method.params[i].pname == "from":
          method.params[i].pname = "_from"
-      if i+1 == len(method.params): 
-         print method.params[i].pname,
-      else:
-         print method.params[i].pname+',',
+      #if i+1 == len(method.params): 
+      print method.params[i].pname,
+      #else:
+      #   print method.params[i].pname+',',
       for enum in enums:
          if enum.c_name == method.params[i].ptype and docs.has_key(enum.c_name):
             enum_list = ""
@@ -95,19 +121,27 @@ def print_functions(spaces, method, docs, enums, level=1):
       print spaces+spaces2+"\"\"\""
       printdoc(docs[method.c_name].description),
       for param in docs[method.c_name].params:
-         if level == 1:
+         if param[0] in ignored_params:
             continue
-         #if level == 1 and param == docs[method.c_name].params[0]:
+         #if level == 1:
          #   continue
+         if level == 1 and param == docs[method.c_name].params[0]:
+            continue
          if param[0] == "from":
             param_name = "_from"
          else:
             param_name = param[0]
-         if enum_str.has_key(param[0]):
-            printdoc('@param '+param_name+': '+param[1]+ "Expected values: "+enum_str[param[0]])
-         else:
-            printdoc('@param '+param_name+': '+param[1])
+         # find original datatype
+         ptype = ""
+         for p in method.params:
+            if p.pname == param_name:
+               ptype = " I{("+ replace_datatype(p.ptype)+")} "
+               break
          # check for enum
+         if enum_str.has_key(param[0]):
+            printdoc('@param '+param_name+': '+param[1].rstrip()+ "Expected values: "+enum_str[param[0]].rstrip()+ptype)
+         else:
+            printdoc('@param '+param_name+': '+param[1].rstrip()+ptype)
       if len(docs[method.c_name].ret) > 0:
          printdoc("@return:"+docs[method.c_name].ret)
       print spaces+spaces2+"\"\"\""
@@ -134,6 +168,9 @@ def add_override_methods(parser):
       else:
          # add as functions    
          parser.define_function(d, ('c-name', d), tuple(params)) 
+
+def func_cmp(x, y):
+   return cmp(x.name, y.name)
    
 # Add codegen path to our import path
 i = 1
@@ -156,12 +193,6 @@ del codegendir
 # Load it
 import docextract, defsparser, override, definitions, string
 
-doc_dirs = string.split("../../app ../../libdraw ../../libgwyddion ../../libgwydgets ../../libgwymodule ../../libprocess .")
-
-
-defs_file = "pygwy.defs"
-wrap_file = "pygwywrap.c"
-ignore_functions = ["static"]
 
 docs = docextract.extract(doc_dirs)
 
@@ -192,7 +223,8 @@ for enum in p.enums:
    print enum.values[len(enum.values)-1][1].replace("GWY_", ""), " = range(", len(enum.values), ")"
 """   
 
-print "# This is dummy GENERATED file used for generation of documentation"
+# Keep GENERATED seprarated so that this file is not marked generated.
+print "# This is dummy " + "GENERATED" + " file used for generation of documentation"
 
 #objects/classes
 
@@ -202,6 +234,7 @@ for obj in p.objects:
       sys.stderr.write("class "+obj.c_name+" not found.\n")
       continue
    #print dir(obj)
+   
    methods = p.find_methods(obj)
    if not methods:
       continue
@@ -231,6 +264,15 @@ for obj in p.objects:
          continue
       if method.varargs:
          continue
+      # skip ignore suffix
+      to_ignore = False
+      for end in ignore_endswith:
+         if method.name.endswith(end):
+            to_ignore = True
+            break
+      if to_ignore:
+         continue
+         
       if wrap_file_cont.find(method.c_name) == -1 and override_file_cont.find(method.c_name) == -1:
          sys.stderr.write("method "+method.c_name+" not found.\n")
          method.name = "UNIMPLEMENTED_"+method.name
@@ -244,6 +286,7 @@ for obj in p.objects:
       print_functions("     ", method, docs, p.enums)
    
 # functions
+p.functions.sort(func_cmp)
 for func in p.functions:
    if isinstance(func, definitions.MethodDef):
       continue
@@ -258,8 +301,17 @@ for func in p.functions:
    # skip ignore functions
    if func.name in ignore_functions:
       continue
+   # skip ignore suffix
+   to_ignore = False
+   for end in ignore_endswith:
+      if func.name.endswith(end):
+         to_ignore = True
+         break
+   if to_ignore:
+      continue
+   # check if function is used in pygwywrap.c file
    if wrap_file_cont.find(func.c_name) == -1 and override_file_cont.find(func.c_name):
       sys.stderr.write("Func "+func.c_name+" not found.\n")
-      func.name = "UNIMPLEMENTED_"+func.name
+      func.name = "z_UNIMPLEMENTED_"+func.name
    print_functions("     ", func, docs, p.enums, 0)
 
